function initBoard(groups, features) {
    var container, child, grandchild, pinFeatures = [];
    
    container = $("#features");
    for (var group in groups) {
        if (groups.hasOwnProperty(group)) {
	    child = $("<li></li>");
	    child.text(groups[group].description).addClass("group-"+group+" list-group-item list-group-item-"+groups[group].cssClass);

	    var nesting = $("<ul></ul>");
	    nesting.appendTo(child);

	    for (var feature in features) {
		if (features[feature].group != group)
		    continue;

		grandchild = $("<li></li>");
		grandchild.text(features[feature].description).addClass("function-"+feature+" list-group-item-"+features[feature].cssClass);
		grandchild.data("feature", feature);
		if (features[feature].help) {
                    grandchild.append('<span class="badge" title="'+features[feature].help+'">?</span>');
		}
		grandchild.appendTo(nesting);
		grandchild.draggable({
                    revert: "invalid",
                    helper: function(event) {
			return $("<div class='dragobj'></div>");
                    },
                    cursorAt: { top: 3, left: 3 }
		});
            }

	    child.appendTo(container);
	}
    }
    
    for (var i=window.board.minPin; i<=window.board.maxPin; i++) {
        pinFeatures[i] = [];
    }
    for (var feature in features) {
        if (features.hasOwnProperty(feature)) {
            for (var j=0; j<features[feature].configurations.length; j++) {
                var pins = features[feature].configurations[j].pins;
                for (p=0; p<pins.length; p++) {
                    pinFeatures[pins[p]].push(feature);
                }
            }
        }
    }
    
    container = $("#board");
    for (var i=window.board.minPin; i<=window.board.maxPin; i++) {
        child = $("<div></div>");

        if (window.board.id == 'Armadillo-640') {
	    /*
	     * pin number
	     *   1-28(28pins): CON9
	     *  29-30( 2pins): CON8
	     *  31-34( 4pins): CON14
	     *  35-84(50pins): CON11
	     */
            if (i<29) {
		child.text(i).addClass("pin pin-"+i);
            } else if (i<31)  {
		child.text(i-28).addClass("pin pin-"+i);
            } else if (i<35)  {
		child.text(i-30).addClass("pin pin-"+i);
            } else {
		child.text(i-34).addClass("lcd pin-"+i);
            }
        } else if (window.board.id == 'Armadillo-610') {
        /*
         * pin number
         * 1-100(100pins): CON2
         * */
            child.text(i).addClass("ext pin-"+i);
        } else if (window.board.id == 'Armadillo-IoT_A6') {
            child.text(i).addClass("pin pin-"+i);
        } else
            child.text(i).addClass("pin pin-"+i);

        child.data("pin", i);
        child.appendTo(container);
        child.click(showPropertyList);
        child.droppable({
            accept: function(draggableElement) {
                var thisPin = $(this).data("pin"),
                    thisFeature = draggableElement.data("feature"),
                    acceptable = pinFeatures[thisPin];
                    
                return acceptable.indexOf(thisFeature) != -1;
            },
            activeClass: "ui-state-default",
            hoverClass: "ui-state-hover",
            tolerance: "pointer",
            drop: function( event, ui ) {
                var thisFeature = ui.draggable.data("feature"),
                    thisPin = $(this).data("pin"),
                    configurations = features[thisFeature].configurations,
                    configuration;
                
                // lookup for the desired pin configuration
                for (var j=0; j<configurations.length; j++) {
                    var pins = configurations[j].pins;
                    for (p=0; p<pins.length; p++) {
                        if (pins[p] == thisPin) {
                            configuration = pins;
                        }
                    }
                }
                
                // are all the pins of this configuration free?
                var pinBusy, configAvailable = true;
                for (p=0; p<configuration.length; p++) {
                    pinBusy = $(".pin-" + configuration[p]).hasClass("pin-busy");
                    if (pinBusy) {
                        configAvailable = false;
                    }
                }
                
                if (!configAvailable) {
                    alert("Cannot overlap pinmuxing! To remove a feature right-click it.");
                    return false;
                }
                
                // mark pins as busy
                for (p=0; p<configuration.length; p++) {
                    var text = ui.draggable.data("feature");
                    if (text == "reg") {
                        text += "_" + thisPin;
                    }
                    $(".pin-" + configuration[p])
                    .text(text)
                    .attr("title", ui.draggable.data("feature"))
                    .addClass("pin-busy pin-busy-used");
                }
            }
        });
    }
};

function getUsedPins()
{
    var pins = [], pin;
    
    for (var i=0; i<=window.board.maxPin; i++) {
        pin = $(".pin-" + i);
        if (pin.hasClass("pin-busy")) {
            pins.push(pin.data("pin"));
        }
    }

    return pins;
}

function dumpConfiguration()
{
    var devices = {}, pin, name;
    
    for (var i=0; i<=window.board.maxPin; i++) {
        pin = $(".pin-" + i);
        if (pin.hasClass("pin-busy")) {
            name = pin.text();
            if (name == '1wire') {
                name = "onewire" + pin.data("pin");
            }
            if (!devices[name]) {
                devices[name] = [];
            }
            
            devices[name].push(pin.data("pin"));
            
            if (board.features[name] && board.features[name].depends && board.features[name].depends instanceof Array) {
                for (var d=0; d<board.features[name].depends.length; d++) {
                    var depend = board.features[name].depends[d];
                    devices[depend] = [];
                }
            }
        }
    }

    return devices;
}

function dumpSNVSPins()
{
    var pins = [];
    if (window.board.snvsPins != undefined) {
        for (var i = 0; i < window.board.snvsPins.length; i++) {
            pins.push(window.board.snvsPins[i]);
        }
    }
    return pins;
}

function applyConfiguration(devices)
{
    for (var device in devices) {
        if (devices.hasOwnProperty(device)) {
            var configuration = devices[device], devname = device;
            
            if (device.substr(0, 7) == 'onewire') {
                devname = '1wire';
            }
            if (device.substr(0, 3) == 'reg') {
                devname = 'reg';
            }
            
            for (p=0; p<configuration.length; p++) {
                var text = devname;
                if (text == 'reg') {
                    text += '_' + configuration[p];
                }
                var pin = $(".pin-" + configuration[p]).text(text)
                if (window.board.features[devname] === undefined) {
                    pin.addClass("pin-busy pin-busy-fixed");
                } else {
                    pin.addClass("pin-busy pin-busy-used");
                }
            }
        }
    }
}

function resetConfiguration()
{
    if (window.board.id == 'Armadillo-640') {
        /*
         * pin number
         *   1-28(28pins): CON9
         *  29-30( 2pins): CON8
         *  31-34( 4pins): CON14
         *  35-84(50pins): CON11
         */
        for (var i=0; i<=window.board.maxPin; i++) {
            if (i<29) {
                $(".pin-" + i).text(i).attr("title", "").removeClass("pin-busy pin-busy-fixed pin-busy-used");
            } else if (i<31)  {
                $(".pin-" + i).text(i-28).attr("title", "").removeClass("pin-busy pin-busy-fixed pin-busy-used");
            } else if (i<35)  {
                $(".pin-" + i).text(i-30).attr("title", "").removeClass("pin-busy pin-busy-fixed pin-busy-used");
            } else {
                $(".pin-" + i).text(i-34).attr("title", "").removeClass("pin-busy pin-busy-fixed pin-busy-used");
            }
        }
    } else if (window.board.id == 'Armadillo-610') {
        /*
         * pin number
         * 1-100(100pins): CON2
         * */
        for (var i=0; i<=window.board.maxPin; i++) {
            $(".pin-" + i).text(i).attr("title", "").removeClass("pin-busy pin-busy-fixed pin-busy-used");
        }
    } else if (window.board.id == 'Armadillo-IoT_A6') {
        for (var i=window.board.minPin; i<=window.board.maxPin; i++) {
            $(".pin-" + i).text(i).attr("title", "").removeClass("pin-busy pin-busy-fixed pin-busy-used");
        }
    } else {
        for (var i=0; i<=window.board.maxPin; i++) {
            $(".pin-" + i).text(i).attr("title", "").removeClass("pin-busy pin-busy-fixed pin-busy-used");
        }
    }

    applyConfiguration(JSON.parse(board.defconfig));
    window.board.properties = {};

    // remove properties table
    $("#properties").empty();
}

function saveConfiguration()
{
    $("body").mask("Please, wait...");
    var properties;
    if (window.board.properties === undefined) {
        properties = {};
    } else {
        properties = window.board.properties;
    }

    $.ajax("/save", {
        data: {
            id: window.board.id,
            conf: JSON.stringify(dumpConfiguration()),
            prop: JSON.stringify(properties),
            snvspins: JSON.stringify(dumpSNVSPins())
        },
        method: 'POST',
        success: function(response) {
            $("body").unmask();
            
            if (response.success) {
                var html = [
                    "Device tree built!"
                ];
                $('#modal-msg').html(html.join(''));
                $('.modal').modal('show');
            } else {
                $('#modal-msg').html("Error: " + (response.message||"unknown error."));
                $('.modal').modal('show');
            }
        },
        error: function() {
            $("body").unmask();
            $('#modal-msg').html("Could not build DTB.");
            $('.modal').modal('show');
        }
    });
}

function applyProperties(event) {
    if (window.board.properties === undefined) {
        window.board.properties = {};
    }

    var feature = event.data.feature;
    var properties = {};
    for (var id of event.data.ids) {
        var propName = $("#" + id + "-name").val();
        var propVal = $("#" + id + "-val").val();
        properties[propName] = propVal;
    }
    window.board.properties[feature] = properties;
}

function showPropertyList(event) {
    $("#properties").empty();
    $('#prop-apply-btn').off('click');

    if ($(event.target).hasClass("pin-busy")) {
        // list device tree properties
        var thisFeature = $(event.target).text();
        var savedFeature = "";
        var properties = window.board.properties;
        var defproperties = window.board.defproperties;
        var container = $("#properties");
        var idx = 0;
        var ids = [];

        savedFeature = thisFeature;
        if (thisFeature.substr(0, 3) == "reg") {
            thisFeature = "reg";
        }

        for (var propName in defproperties[thisFeature]) {
            var displayName = propName.split("/").pop();
            var li = $("<li></li>");
            li.addClass(thisFeature);
            li.text(displayName + " = ");

            if (Array.isArray(defproperties[thisFeature][propName])) {
                var selectVal = $("<select></select>").attr({id: thisFeature + "-" + idx + "-val"});
                for (var propVal of defproperties[thisFeature][propName]) {
                    var optionVal = $("<option></option>").val(propVal).text(propVal);
                    selectVal.append(optionVal);
                }
                if (properties[savedFeature] !== undefined && properties[savedFeature][propName] !== undefined) {
                    selectVal.val(properties[savedFeature][propName]);
                }
                selectVal.appendTo(li);
            } else {
                var inputVal = $("<input/>");
                var propVal = defproperties[thisFeature][propName];
                if (properties[savedFeature] !== undefined && properties[savedFeature][propName] !== undefined) {
                    propVal = properties[savedFeature][propName];
                }
                inputVal.val(propVal)
                .attr({type: "text", id: thisFeature + "-" + idx + "-val"});
                inputVal.appendTo(li);
            }

            var inputPropName = $("<input/>");
            inputPropName.val(propName)
            .attr({type: "hidden", id: thisFeature + "-" + idx + "-name"});
            inputPropName.appendTo(li);
            li.appendTo(container);
            
            ids.push(thisFeature + "-" + idx++);
        }
        if (defproperties[thisFeature] !== undefined) {
            var li = $("<li></li>");
            li.addClass(thisFeature);
            var ahref = $("<a></a>");
            ahref.text("Apply").attr({href: "#", id: "prop-apply-btn"});
            ahref.appendTo(li);
            li.appendTo(container);
            $('#prop-apply-btn').on('click', {feature: savedFeature, ids: ids}, applyProperties);
        }
    }
}

$(function() {
    $.material.init();
    
    $(document).contextmenu({
        delegate: [".pin", ".lcd"],
        autoFocus: true,
        preventContextMenuForPopup: true,
        preventSelect: true,
        taphold: true,
        menu: [
            {title: "Remove", cmd: "remove", uiIcon: "ui-icon-scissors"}
        ],
        select: function(event, ui) {
            var thisFeature = ui.target.text(),
            savedFeature = thisFeature;
            if (thisFeature.substr(0, 3) == "reg") {
                thisFeature = "reg";
            }
            var  thisPin = ui.target.data("pin"),
                configurations = window.board.features[thisFeature].configurations,
                configuration;

            // lookup for the desired pin configuration
            for (var j=0; j<configurations.length; j++) {
                var pins = configurations[j].pins;
                for (p=0; p<pins.length; p++) {
                    if (pins[p] == thisPin) {
                        configuration = pins;
                    }
                }
            }

            // mark pins as free
            for (p=0; p<configuration.length; p++) {
                $(".pin-" + configuration[p])
                .text(configuration[p])
                .attr("title", "")
                .removeClass("pin-busy pin-busy-used pin-busy-fixed");
            }

            // delete properties
            if (window.board.properties[savedFeature] !== undefined) {
                delete window.board.properties[savedFeature];
            }
            $("." + thisFeature).remove()
        },
        beforeOpen: function(event, ui) {
            ui.menu.zIndex( $(event.target).zIndex() + 1);
            $(document).contextmenu("enableEntry", "remove", ui.target.hasClass("pin-busy"));
        }
    });
    
    $('.board-name').html(window.board.name);
    initBoard(window.board.groups, window.board.features);
    
    $('#reset-btn').on('click', resetConfiguration);
    $('#save-btn').on('click', saveConfiguration);
});
